@@ -14,11 +14,10 @@
 #  under the License.
 #
 
-import requests
-
-from neutron_lib.plugins import directory
-
 from neutron.db import api as db_api
+from neutron_lib.db import api as lib_db_api
+from neutron_lib.plugins import directory
+import requests
 
 from networking_odl.common import client
 from networking_odl.common import constants as odl_const
@@ -79,21 +78,21 @@ def register(driver, resources, handler=None):
         FULL_SYNC_RESOURCES[resource] = handler
 
 
-@db_api.retry_if_session_inactive()
+@lib_db_api.retry_if_session_inactive()
+@db_api.context_manager.writer.savepoint
 def full_sync(context):
-    with db_api.autonested_transaction(context.session):
-        if not _full_sync_needed(context):
-            return
+    if not _full_sync_needed(context):
+        return
 
-        db.delete_pending_rows(context.session, _OPS_TO_DELETE_ON_SYNC)
+    db.delete_pending_rows(context, _OPS_TO_DELETE_ON_SYNC)
 
-        for resource_type in _ORDERED_ODL_RESOURCES:
-            handler = FULL_SYNC_RESOURCES.get(resource_type)
-            if handler:
-                _sync_resources(context, resource_type, handler)
+    for resource_type in _ORDERED_ODL_RESOURCES:
+        handler = FULL_SYNC_RESOURCES.get(resource_type)
+        if handler:
+            _sync_resources(context, resource_type, handler)
 
-        journal.record(context, odl_const.ODL_NETWORK, _CANARY_NETWORK_ID,
-                       odl_const.ODL_CREATE, _CANARY_NETWORK_DATA)
+    journal.record(context, odl_const.ODL_NETWORK, _CANARY_NETWORK_ID,
+                   odl_const.ODL_CREATE, _CANARY_NETWORK_DATA)
 
 
 def _full_sync_needed(context):
@@ -120,7 +119,7 @@ def _canary_network_missing_on_odl():
 
 def _canary_network_not_in_journal(context):
     return not db.get_pending_or_processing_ops(
-        context.session, _CANARY_NETWORK_ID, operation=odl_const.ODL_CREATE)
+        context, _CANARY_NETWORK_ID, operation=odl_const.ODL_CREATE)
 
 
 def get_resources_require_id(plugin, context, get_resources_for_id,
@@ -150,14 +149,14 @@ def _sync_resources(context, object_type, handler):
                        odl_const.ODL_CREATE, resource)
 
 
-@db_api.retry_if_session_inactive()
+@lib_db_api.retry_if_session_inactive()
 # TODO(rajivk): Change name from sync_resource to _sync_resources
 # once, we are completely moved to new sync mechanism to plug new syncing
 # mechanism.
 def sync_resources(context, resource_type):
     driver = base_driver.get_driver(resource_type)
     resources = driver.get_resources_for_full_sync(context, resource_type)
-    with db_api.autonested_transaction(context.session):
+    with db_api.context_manager.writer.savepoint.using(context):
         for resource in resources:
             journal.record(context, resource_type, resource['id'],
                            odl_const.ODL_CREATE, resource)
