@@ -55,11 +55,11 @@ class OpenDaylightBgpvpnDriverTestCase(base_v2.OpenDaylightConfigBase):
         return fake_net_assoc
 
     def _assert_op(self, operation, object_type, data, precommit=True):
-        rows = sorted(db.get_all_db_rows_by_state(self.db_session,
+        rows = sorted(db.get_all_db_rows_by_state(self.db_context,
                                                   odl_const.PENDING),
                       key=lambda x: x.seqnum)
         if precommit:
-            self.db_session.flush()
+            self.db_context.session.flush()
             self.assertEqual(operation, rows[0]['operation'])
             self.assertEqual(object_type, rows[0]['object_type'])
             self.assertEqual(data['id'], rows[0]['object_uuid'])
