@@ -17,6 +17,7 @@ import mock
 from neutron.db import api as db_api
 
 from networking_odl.common import constants as odl_const
+from networking_odl.common import odl_features
 from networking_odl.db import db
 from networking_odl.qos import qos_driver_v2 as qos_driver
 from networking_odl.tests import base
@@ -28,12 +29,38 @@ class OpenDaylightQosDriverTestCase(base_v2.OpenDaylightConfigBase):
     def setUp(self):
         self.useFixture(base.OpenDaylightJournalThreadFixture())
         super(OpenDaylightQosDriverTestCase, self).setUp()
-        self.qos_driver = qos_driver.OpenDaylightQosDriver()
+        self.qos_driver = qos_driver.OpenDaylightQosDriver({})
+        self.addCleanup(odl_features.deinit)
 
-    def _get_mock_context(self):
+    def test_qos_supported_rules_are_fetched_from_odl_feature(self):
+        feature_json = """{"features": {"feature":
+                                [{"service-provider-feature":
+                                "neutron-extensions:operational-port-status"},
+                                {"service-provider-feature":
+                                "neutron-extensions:qos-rules",
+                                "configuration": {"key": "value"}}]}}"""
+
+        self.cfg.config(odl_features_json=feature_json, group='ml2_odl')
+        odl_features.init()
+
+        qos_driver_object = qos_driver.OpenDaylightQosDriver.create()
+
+        self.assertDictEqual(qos_driver_object.supported_rules,
+                             {'key': 'value'})
+
+    def test_default_values_for_supported_rules(self):
+        self.cfg.config(odl_features='key', group='ml2_odl')
+        odl_features.init()
+
+        qos_driver_object = qos_driver.OpenDaylightQosDriver.create()
+
+        self.assertDictEqual(qos_driver_object.supported_rules,
+                             qos_driver.DEFAULT_QOS_RULES)
+
+    def _get_mock_context(self, session=None):
         current = {'tenant_id': 'tenant_id'}
         context = mock.Mock(current=current)
-        context.session = self.db_session
+        context.session = session
         return context
 
     def _get_mock_qos_operation_data(self):
@@ -50,19 +77,19 @@ class OpenDaylightQosDriverTestCase(base_v2.OpenDaylightConfigBase):
 
     def _call_operation_object(self, operation, object_type):
         qos_data = self._get_mock_qos_operation_data()
-        context = self._get_mock_context()
         method = getattr(self.qos_driver, '%s_%s' % (operation,
                                                      object_type))
 
         assert object_type.endswith("precommit")
-        with db_api.context_manager.writer.using(context):
+        with db_api.context_manager.writer.using(self.db_context):
+            context = self._get_mock_context(self.db_context.session)
             method(context, qos_data)
 
     def _test_qos_policy(self, operation):
         self._call_operation_object(operation=operation,
                                     object_type='policy_precommit')
         qos_data = self._get_mock_qos_operation_data()
-        row = db.get_oldest_pending_db_row_with_lock(self.db_session)
+        row = db.get_oldest_pending_db_row_with_lock(self.db_context)
         self.assertEqual(operation, row['operation'])
         self.assertEqual(qos_data.to_dict()['id'], row['object_uuid'])
 
